#!/usr/bin/perl

# postbank.pl
# -----------
#
# Kleines Perl-Skript zum Abrufen des Kontostands im Postbank-Online-Banking.
# Kann leicht modifiziert werden, um Umstze abzurufen oder sogar berweisungen
# zu machen.
# 
# Bentigt LWP und die SSL-Untersttzung (libcrypt-ssleay-perl) dafr!
#
# Um mit dem Session Management auf Nummer sicher zu gehen, ruft das Skript
# artig wie ein normaler Browser die Seiten einzeln hintereinander ab.
#
# USE AT YOUR OWN RISK!
#
# (c) Frederik Ramm <frederik@remote.org> 2002
# Aktuellste Version hier:
# http://www.remote.org/frederik/projects/software/banking/
# Version 1.0
# Public Domain.

use strict;
use HTTP::Request;
use LWP::UserAgent;

my $kto = 0123456; # hier die eigene Kontonummer
my $pin = 01234; # hier die pin

# --- STAGE 1 ---
#
# Die Startseite abrufen.
my $request = HTTP::Request->new("GET", "http://www.postbank-banking.de/");
my $ua = LWP::UserAgent->new;
my $response = $ua->simple_request($request);

# Dabei gibt's in der Regel einen Redirect mit "location"-Header,
# dem wir folgen.

my $loc = $response->headers->{"location"};
die "not received location header" unless defined($loc);

# --- STAGE 2 ---

$request = HTTP::Request->new("GET", $loc);
$response = $ua->simple_request($request);

die $response->error_as_HTML . " at stage 2" unless ($response->is_success);

# Die Seite, die nun kommt, ist ein Frameset, aus dem wir uns den 
# Content-Frame herausklauben.
#
die "cannot find content frame at stage 2" 
    unless ($response->content =~ /<frame[^>]+src="([^"]+)"[^>]+name="contents"/);
my $newl = $1;

# --- STAGE 3 ---

$loc = makeAbsolute($loc, $newl);
$request = HTTP::Request->new("GET", $loc);
$response = $ua->simple_request($request);
# hier $response->content in ein File dumpen, wenn man es untersuchen will:
# open(FILE,">response.html");print(FILE $response->content);close(FILE);

die "error in stage 3" unless ($response->is_success);

# Hier muss jetzt ein Formular zum Login kommen.

die "no form in stage 3" unless ($response->content =~ /(<form.*<\/form>)/si);
my $form = $1;

die "cannot understand form in stage 3(a)" 
    unless $form =~ /<form[^>]*method\s*=\s*["']?([a-z]+)["']?/si; 
my $method = $1;
$method=~y/a-z/A-Z/;

die "cannot understand form in stage 3(b)" 
    unless $form =~ /<form[^>]*action\s*=\s*(\S+)/si;
my $action = $1;
$action =~ y/"'>//d;

# <input type=hidden>-Felder suchen 
my $vars="";
while($form =~ /(<input[^>]*type\s*=\s*["']?hidden["']?[^>]*>)(.*)/si)
{
    my $inp=$1;
    $form=$2;
    die "cannot understand input tag in stage 3(c)"     
        unless ($inp =~ /name\s*=\s*(\S+)/);
    my $name = $1;
    $name =~ y/"'>//d;
    die "cannot understand input tag in stage 3(c)" 
        unless ($inp =~ /value\s*=\s*(\S+)/);
    my $value = $1;
    $value =~ y/"'>//d;
    $vars .= "&" if ($vars ne "");
    $vars.=$name."=".$value;
}

# hier knnte man jetzt auch eine andere FUNCTION nehmen, wenn man z.B. direkt
# zu einem berweisungsformular will.
#
$vars.= "&Kontonummer=$kto&PIN=$pin&FUNCTION=ACCOUNTBALANCE&LOGIN.x=26&LOGIN.y=11";

$loc = makeAbsolute($loc, $action);
$request = HTTP::Request->new($method, $loc);
$request->content($vars);
$request->push_header("Content-type" => "application/x-www-form-urlencoded");

# --- STAGE 4 ---
#
# Schlielich jetzt der Login, zurck sollte der Kontostand kommen
$response = $ua->simple_request($request);
die "http error int stage 4" unless ($response->is_success);

my $res = $response->content;

# innerste <table> herausschlen
while ($res =~ /<table(.*)<\/table>/is)
{
    $res = $1;
}
my $found=0;
# und <tr>-Elemente ansehen
while($res =~ /<tr(.*?)<\/tr>(.*)/si)
{
    $res = $2;
    my $tr = $1;
    my @line;
    while ($tr =~ /(<td.*?<\/td>)(.*)/si)
    {
        $tr=$2;
        my $td=$1;
        $td =~ s/<p[^<]*>/ /isg;
        $td =~ s/<br[^<]*>/ /isg;
        $td =~ s/<[^<]+>//g;
        $td =~ s/\s+/ /g;
        $td =~ /^\s*(.*?)\s*$/;
        push(@line, $1) unless ($1 eq "");
    }
    print join(" / ", @line);
    print "\n";
    $found=1;
}
die "no lines found" unless ($found);

# gibt den absoluten URL zurck, der sich ergibt, wenn man das
# 2. Argument mit BASE des 1. evaluiert
sub makeAbsolute
{
    my ($loc, $newl) = @_;
    if ($newl =~ /^\//)
    {
        $loc =~ /^(.*:\/\/[^\/]*)/;
        $loc = $1 . $newl;
    }
    else
    {
        while($newl =~ /^\.\.\/(.*)/)
        {
            $newl = $1;
            $loc =~ /^(.*)(\/(.*))?$/;
            $loc = $1;
        }
        $loc .= "/".$newl;
    }
    return $loc;
}
